<?php
/**
 * The table list class
 * 
 * @link       https://1.envato.market/mgXE4y
 * @since      2.1
 *
 * @package    WordPress
 * @subpackage StreamTube
 * @author     phpface <nttoanbrvt@gmail.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

class StreamTube_Core_Bunny_Item_List extends WP_List_Table {

    private $bunnycdn;

    public function __construct() {
        parent::__construct( array(
            'singular' => esc_html__( 'Video', 'streamtube-core' ),
            'plural'   => esc_html__( 'Videos', 'streamtube-core' ),
            'ajax'     => true
        ) );

        $this->bunnycdn = streamtube_core()->get()->bunnycdn;
    }

    public function get_columns() {
        $columns = array(
            'cb'        => '<input type="checkbox" />',
            'title'     => esc_html__( 'Title', 'streamtube-core' ),
            'thumbnail' => esc_html__( 'Thumbnail', 'streamtube-core' ),
            'guid'      =>  esc_html__( 'GUID', 'streamtube-core' ),
            'category'  =>  esc_html__( 'Category', 'streamtube-core' ),
            'storageSize'   =>  esc_html__( 'Storage Size', 'streamtube-core' ),
            'dateUploaded'  =>  esc_html__( 'Date', 'streamtube-core' ),
            'action'        =>  esc_html__( 'Action', 'streamtube-core' )
        );
        return $columns;
    }

    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name ){
        switch( $column_name ) {
            case 'title':
                return sprintf(
                    '<a class="row-title" target="_blank" href="%s">%s</a>',
                    esc_url( $this->bunnycdn->bunnyAPI->get_direct_player( $item['videoLibraryId'], $item['guid'] ) ),
                    esc_html( $item['title'] )
                );
            break;

            case 'thumbnail':

                return sprintf(
                    '<div class="ratio ratio-16x9"><a target="_blank" href="%s"><img class="img-thumbnail" src="%s"></a></div>',
                    esc_url( $this->bunnycdn->bunnyAPI->get_direct_player( $item['videoLibraryId'], $item['guid'] ) ),
                    $this->bunnycdn->bunnyAPI->get_video_thumbnail_url( $item['guid'] )
                );
            break;

            case 'storageSize':
                return size_format( $item[ $column_name ] );
            break;

            case 'dateUploaded':
                return sprintf(
                    esc_html__( '%s ago', 'streamtube-core' ),
                    human_time_diff( strtotime( $item[ $column_name ] ), current_time( 'U' ) )
                );
            break;

            case 'action':
                if( "" != $attachment_id = $this->bunnycdn->get_post_id_from_videoId( $item['guid'] ) ){
                    printf(
                        '<span class="badge bg-success"><a target="_blank" class="text-white" href="%s">%s</a></span>',
                        esc_url( get_permalink( get_post( $attachment_id )->post_parent ) ),
                        esc_html__( 'Imported', 'streamtube-core' )
                    );
                }else{
                    printf(
                        '<button class="button-primary button-import-bunny-video" data-guid="%s" data-nonce="%s" type="button">%s</button>',
                        esc_attr( $item['guid'] ),
                        wp_create_nonce( $item['guid'] ),
                        esc_html__( 'Import', 'streamtube-core' )
                    );
                }
            break;

            default:
                return $item[ $column_name ];
            break;
        }
    }

    /**
     * Define which columns are hidden
     *
     * @return array
     */
    public function get_hidden_columns() {
        return array();
    }

    /**
     * Define the sortable columns
     *
     * @return array
     */
    public function get_sortable_columns() {
        return array(
            'title'         => array( 'title', false )
        );
    }

    public function get_bulk_actions(){
        $action = array(
            'import'    =>  esc_html__( 'Import', 'streamtube-core' )
        );
        return $action; 
    }    

    /**
     * Render the bulk edit checkbox
     *
     * @param array $item
     *
     * @return string
     */
    function column_cb( $item ) {
        return sprintf(
            '<input type="checkbox" name="guid[]" value="%s" />', $item['guid']
        );
    }    

    /**
     * Get the table data
     *
     * @return array
     */
    private function table_data() {
        
        $search = isset( $_REQUEST['s'] ) ? wp_unslash( $_REQUEST['s'] ) : '';

        $data = $this->bunnycdn->bunnyAPI->get_videos( array(
            'search' => $search,
        ) );

        if( is_wp_error( $data ) ){
            return array();
        }

        return $data['items'];
    }

    /**
     * Sort data based on the selected column and order
     *
     * @param array $a
     * @param array $b
     * @return int
     */
    private function sort_data( $a, $b ) {
        // Set defaults
        $orderby = 'title';
        $order = 'asc';

        // If orderby is set, use this as the sort column
        if ( ! empty( $_GET['orderby'] ) ) {
            $orderby = $_GET['orderby'];
        }

        // If order is set, use this as the order
        if ( ! empty( $_GET['order'] ) ) {
            $order = $_GET['order'];
        }

        $result = strcmp( $a[ $orderby ], $b[ $orderby ] );

        return ( $order === 'asc' ) ? $result : -$result;
    }

    public function prepare_items() {
        $columns  = $this->get_columns();
        $hidden   = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $data = $this->table_data();
        usort( $data, array( $this, 'sort_data' ) );

        $perPage     = 50;
        $currentPage = $this->get_pagenum();
        $totalItems  = count( $data );

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage,
        ) );

        $data = array_slice( $data, ( ( $currentPage - 1 ) * $perPage ), $perPage );

        $this->_column_headers = array( $columns, $hidden, $sortable );
        $this->items = $data;

        $this->process_bulk_action();
    }

    public function process_bulk_action() {

        if( ! current_user_can( 'administrator' ) ){
            return;
        }

        $action = $this->current_action();

        $guids = isset( $_REQUEST['guid'] ) ? wp_unslash( $_REQUEST['guid'] ) : false;

        switch ( $action ) {
            case 'import':
                for ( $i=0;  $i < count( $guids );  $i++) { 

                    $attachment_id = $this->bunnycdn->get_post_id_from_videoId( $guids[$i] );

                    if( ! $attachment_id ){
                        $this->bunnycdn->import_bunny_video( 
                            $guids[$i], 
                            apply_filters( 'streamtube/core/bunny/bulk_import/tax', false ) 
                        );
                    }
                }
            break;
        }
    }
}